#include <stdio.h>
#include <math.h>
#include "war.h"
#include "hash.h"
#include "hashItem.h"

unsigned int FunctionSwitch = 0xffffffff;
int areaWidth = 256, areaHeight = 256;
int initialCountries = 256;
int initialVillages = 4;
int initialVillageSize = 10;
double MutationWidth = 25.5;
Land *theLand;
unsigned long RandSeed;

#define	TempMemUnit	128

void *allocTempMem(int size) {
	static void *tempMem = NULL;
	static int nSize = 0;
	int ns = ((size + TempMemUnit - 1) / TempMemUnit) * TempMemUnit;
	if (nSize < ns) {
		tempMem = (void *)realloc(tempMem, ns);
		if (!tempMem) {
			perror("realloc in allocTempMem");
			exit(0);
		}
		nSize = ns;
	}
	return tempMem;
}
void randomShuffle(void **array, int size) {
	int	i, k;
	void	*x;
	for (i = 0; i < size - 1; i ++) {
		k = (lrand48() % (size - i)) + i;
		if (k > i) {
			x = array[i];
			array[i] = array[k];
			array[k] = x;
		}
	}
}
double gaussianRandom() {
	extern double	drand48();
	double	u = drand48(), v = drand48();
	return sqrt(log(u) * -2.) * cos(M_PI * v * 2.);
}
void mutatedCopy(unsigned char *dest, unsigned char *src, int size) {
	int	i;
	extern double	drand48();
	for (i = 0; i < size; i ++) {
		int	x = src[i] + gaussianRandom() * MutationWidth;
		dest[i] = (x < 0)? 0 : (x > 255)? 255 : x;
	}
}
static void initLand(void) {
	int x, y;
	theLand = (Land *)malloc(sizeof(Land) * areaWidth * areaHeight);
	if (!theLand) { perror("malloc for the land"); exit(0); }
	memset(theLand, 0, sizeof(Land) * areaWidth * areaHeight);
	for (y = 0; y < areaHeight; y ++)
	for (x = 0; x < areaWidth; x ++) {
		Land *p = &theLand[y * areaWidth + x];
		p->x = x; p->y = y;
	}
}
int emptyLand(void *land) {
	return (((Land *)land)->v == NULL);
}
static Land *randomFrontier(Country *c) {
	Land *p = (Land *)randomItemInHash(emptyLand, c->border);
	if (p) return p;
	do {
		int x = lrand48() % areaWidth;
		int y = lrand48() % areaHeight;
		p = &theLand[y * areaWidth + x];
	} while (p->v);
	return p;
}

#define leftLand(l)  theLand[l->y*areaWidth+(l->x+areaWidth-1)%areaWidth]
#define rightLand(l) theLand[l->y*areaWidth+(l->x+1)%areaWidth]
#define upperLand(l) theLand[((l->y+areaHeight-1)%areaHeight)*areaWidth+l->x]
#define lowerLand(l) theLand[((l->y+1)%areaHeight)*areaWidth+l->x]

static checkNewBorder(Village *nv, Village *ov, Land *cl, Land *rl) {
	Country *nc = nv? nv->k : NULL;
	Country *oc = ov? ov->k : NULL;
	Village *v = rl->v;
	Country *c = v? v->k : NULL;
	if (oc != nc) {
		if (nc) {
			if (c != nc) putHash((long)rl, (void *)rl, nc->border);
			else removeHash((long)cl, nc->border);
		}
		if (oc && c == oc) putHash((long)cl, (void *)cl, oc->border);
	}
	if (nv) {
		if (v != nv) putHash((long)rl, (void *)rl, nv->border);
		else removeHash((long)cl, nv->border);
	}
	if (ov && v == ov) putHash((long)cl, (void *)cl, ov->border);
}
void occupyLand(Land *land, Village *v) {
	Village *oldV;
	land->pre = NULL;
	land->post = v->l;
	if (v->l) v->l->pre = land;
	v->l = land;
	oldV = land->v;
	land->v = v;
	v->nLands ++;
	if (v != oldV) {
		checkNewBorder(v, oldV, land, &upperLand(land));
		checkNewBorder(v, oldV, land, &lowerLand(land));
		checkNewBorder(v, oldV, land, &leftLand(land));
		checkNewBorder(v, oldV, land, &rightLand(land));
	}
}
void showLand(Land *l) {
	printf("Land grid (%d,%d)\n", l->x, l->y);
}
static void brightenIfTooDark(MyRGB *c) {
	int br = (int)c->r + c->g + c->b;
	if (br >= 255) return;
	c->r += (255 - c->r) * (255 - br) / (255*3 - br);
	c->g += (255 - c->g) * (255 - br) / (255*3 - br);
	c->b += (255 - c->b) * (255 - br) / (255*3 - br);
}
static void setRandomRGB(MyRGB *c) {
	unsigned long pxl = lrand48();
	c->r = (pxl >> 16) & 0xff;
	c->g = (pxl >> 8) & 0xff;
	c->b = pxl & 0xff;
	brightenIfTooDark(c);
}
static unsigned char similar8bitVal(unsigned char v) {
	int k = v + (lrand48() % 92) - 46;
	return (unsigned char)((k < 0)? 0 : ((k > 255)? 255 : k));
}
void setSimilarRGB(MyRGB *cd, MyRGB *cs) {
	cd->r = similar8bitVal(cs->r);
	cd->g = similar8bitVal(cs->g);
	cd->b = similar8bitVal(cs->b);
	brightenIfTooDark(cd);
}
static int placeNewRandomVillage(Country *c) {
	int	i;
	Village *v;
	Land *l = randomFrontier(c);
	if (!l) return 0;
	v = newVillage();
	v->k = c;
	c->nVillages ++;
	occupyLand(l, v);
	v->post = c->v;
	if (c->v) c->v->pre = v;
	c->v = v;
	setRandomRGB(&v->rgb);
	for (i = 1; i < initialVillageSize; i ++) {
		l = randomItemInHash(emptyLand, v->border);
		if (!l) break;
		occupyLand(l, v);
	}
	return 1;
}
static void createRandomMap(void) {
	int i, j;
	for (i = 0; i < initialCountries; i ++) {
		Country *c = newCountry();
		for (j = 0; j < initialVillages; j ++)
			if (!placeNewRandomVillage(c)) break;
		registarCountry(c);
		setRandomRGB(&c->rgb);
	}
}
// These routines below are refered from splitVillage in vil.c
void traverseLand(Land *l, Village *v, int k) {
	if (l->work || l->v != v) return;
	l->work = k;
	traverseLand(&leftLand(l), v, k);
	traverseLand(&rightLand(l), v, k);
	traverseLand(&upperLand(l), v, k);
	traverseLand(&lowerLand(l), v, k);
}
static int checkVillageBorder(Land *l, Village *v) {
	return (leftLand(l).v  == v || rightLand(l).v == v ||
		upperLand(l).v == v || lowerLand(l).v == v);
}
void checkNewVilBorder(Village *v1, Village *v2) {
	Land	*l;
	for (l = v1->l; l; l = l->post)
		if (checkVillageBorder(l, v2))
			putHash((long)l, (void *)l, v2->border);
}
struct villages { Village *v, *nv; };
void checkSplitBorder(HashItem *cell, struct villages *vs) {
	Land *l = (Land *)cell->item;
	if (l->v != vs->nv && checkVillageBorder(l, vs->nv))
		putHash((long)l, (void *)l, vs->nv->border);
	if (!checkVillageBorder(l, vs->v))
		removeHashCell((void *)cell, vs->v->border);
}
struct countries { Country *oldC, *newC; };
void checkCBorderNew(HashItem *cell, Village *v) {
	if (((Land *)cell->item)->v == v)
		 removeHashCell((void *)cell, v->k->border);
}
static int vc(Village *v, Country *c) {
	return (v && v->k == c);
}
static int checkCountryBorder(Land *l, Country *c) {
	return (vc(leftLand(l).v,c) || vc(rightLand(l).v,c) ||
		vc(upperLand(l).v,c) || vc(lowerLand(l).v,c));
}
void checkCBorderSub(Land *l, struct countries *cc) {
	if (!l->v || l->v->k != cc->newC)
		putHash((long)l, (void *)l, cc->newC->border);
	if ((!l->v || l->v->k != cc->oldC) && !checkCountryBorder(l, cc->oldC))
		removeHash((long)l, cc->oldC->border);
}
struct cAndV { Country *c; Village *v; };
static int vcr(Village *v, struct cAndV *cv) {
	return (v && v != cv->v && v->k == cv->c);
}
static int checkCountryBorderRV(Land *l, struct cAndV *cv) {
	return (vcr(leftLand(l).v,cv) || vcr(rightLand(l).v,cv) ||
		vcr(upperLand(l).v,cv) || vcr(lowerLand(l).v,cv));
}
void checkCBorderRVSub(Land *l, struct cAndV *cv) {
	if ((!l->v || l->v->k != cv->c) && !checkCountryBorderRV(l, cv));
		removeHash((long)l, cv->c->border);
}
// refered from checkSepAndRuin in country.c
struct cs_rec { int n; Country **cs; };
void checkSepCountryBorder(HashItem *cell, struct cs_rec *csp) {
	int i;
	Land *l = (Land *)cell->item;
	for (i = 1; i < csp->n; i ++)
		if (checkCountryBorder(l, csp->cs[i]))
			putHash((long)l, (void *)l, csp->cs[i]->border);
	if (!checkCountryBorder(l, csp->cs[0]))
		removeHashCell(cell, csp->cs[0]->border);
}
void restartWorld(void) {
	srand48(RandSeed);
	releaseAllCountries();
	createRandomMap();
	showMap();
	resetCountryLog();
}
void resetWorld(void) {
	TracedCountryID = TracedVillageID = 0;
	RandSeed = time(0) + drand48();
	restartWorld();
}
main(int argc, char *argv[]) {
	if (argc < 2) RandSeed = 20030220ul;
	else if ((RandSeed = atoi(argv[1])) == 0) RandSeed = time(0);
	initLand();
	initUserIF(argc, argv);
	restartWorld();
	loopUserIF();
	return 0;
}
