#include <math.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>
#include "war.h"
extern Display	*display;
extern GC	gc;
extern Widget	MainWindow;
static Widget	Pshell = 0, paramBtn;
extern void reserveProc(void (*)(void));

typedef struct {
	char	*name;
	double	*v;
	double	min, max, mag;
	Widget	scale;
	double	defaultVal;
} ParamDouble;

ParamDouble theDoubleParams[] = {
	{ "FoodProductionPerWorker", &FoodProductionPerWorker, 1., 5., 100.},
	{ "FoodSavingRate", &FoodSavingRate, 0., 1., 100. },
	{ "PopulationIncreasingRate", &PopulationIncreasingRate, 1., 2., 100. },
	{ "OfficerA", &OfficerA, 0., 1., 100. },
	{ "OfficerB", &OfficerB, 0., 200., 1. },
	{ "FluctuationOfPopulation", &FluctuationOfPopulation, 0., 0.2, 1000.},
	{ "FluctuationOfProduction", &FluctuationOfProduction, 0., 0.2, 1000.},
	{ "MutationWidth", &MutationWidth, 0., 200., 1. },
	{ NULL }
};

typedef struct {
	char	*name;
	unsigned int	flagBit;
} ParamSwitch;

ParamSwitch theSwitches[] = {
	{ "Foods Dlivery", FnDelivery },
	{ "Invasion", FnWar },
	{ "Independence", FnIndependence },
	{ "Move country", FnMoveCountry },
	{ "Regular Mutation", FnRegularMutate },
	{ NULL }
};

static void ScaleDoubleCB(Widget w, XtPointer cl, XtPointer cd) {
	ParamDouble *pd = (ParamDouble *)cl;
	XmScaleCallbackStruct *sc = (XmScaleCallbackStruct *)cd;
	*pd->v = sc->value / pd->mag;
	printf("%s <- %g\n",pd->name,*pd->v);
}
static void ResetValuesCB(Widget w, XtPointer cl, XtPointer cd) {
	ParamDouble *pd;
	for (pd = theDoubleParams; pd->name; pd ++) {
		*pd->v = pd->defaultVal;
		XtVaSetValues(pd->scale,XmNvalue,(int)(*pd->v * pd->mag),NULL);
	}
}
static void SwitchCB(Widget w, XtPointer cl, XtPointer cd) {
	ParamSwitch *ps = (ParamSwitch *)cl;
	Boolean value;
	XtVaGetValues(w,XmNset,&value,NULL);
	if (value) FunctionSwitch |= ps->flagBit;
	else FunctionSwitch &= ~ ps->flagBit;
}
static void createParamShell(void) {
	int	n;
	Widget	main, rc, button;
	ParamDouble *pd;
	ParamSwitch *ps;
	Arg	args[7];
	Pshell = XtCreatePopupShell("Parameter",
		transientShellWidgetClass,MainWindow,NULL,0);
	XtManageChild(main=XmCreateForm(Pshell,"main",NULL,0));
	XtManageChild(rc=XmCreateRowColumn(main,"sliders",NULL,0));
	XtSetArg(args[0],XmNorientation,XmHORIZONTAL);
	XtSetArg(args[1],XmNshowValue,True);
	for (pd = theDoubleParams; pd->name; pd ++) {
		n = 2;
		XtSetArg(args[n],XmNminimum,(int)(pd->min * pd->mag)); n ++;
		XtSetArg(args[n],XmNmaximum,(int)(pd->max * pd->mag)); n ++;
		XtSetArg(args[n],XmNvalue,(int)(*pd->v * pd->mag)); n ++;
		XtSetArg(args[n],XmNdecimalPoints,(short)log10(pd->mag)); n ++;
		XtSetArg(args[n],XmNtitleString,
			XmStringCreateSimple(pd->name)); n ++;
		XtManageChild(pd->scale=XmCreateScale(rc,pd->name,args,n));
		XtAddCallback(pd->scale,XmNvalueChangedCallback,
			ScaleDoubleCB,(XtPointer)pd);
		pd->defaultVal = *pd->v;
	}

	XtManageChild(rc=XmCreateRowColumn(main,"switches",NULL,0));
	XtManageChild(button=XmCreatePushButton(rc,"default",NULL,0));
	XtAddCallback(button,XmNactivateCallback,ResetValuesCB,NULL);
	XtSetArg(args[0],XmNset,True);
	for (ps = theSwitches; ps->name; ps ++) {
		XtManageChild(button=XmCreateToggleButton(rc,ps->name,args,1));
		XtAddCallback(button,XmNvalueChangedCallback,
			SwitchCB,(XtPointer)ps);
	}
	XtPopup(Pshell,XtGrabNone);
}
static void openParamWindow(void) {
	if (!Pshell) createParamShell();
	else XtPopup(Pshell,XtGrabNone);
}
static void closeParamWindow(void) {
	XtPopdown(Pshell);
}
static void OpenParamCB(Widget w, XtPointer cl, XtPointer cd) {
	Boolean value;
	XtVaGetValues(w,XmNset,&value,NULL);
	if (value) reserveProc(openParamWindow);
	else reserveProc(closeParamWindow);
}
void setupMenuItemParam(Widget menu) {
	XtManageChild(paramBtn=XmCreateToggleButton(menu,"Parameters",NULL,0));
	XtAddCallback(paramBtn,XmNvalueChangedCallback,OpenParamCB,NULL);
}
